//Be sure to include STCOMM.JAR from the Sametime toolkit as a referenced JAR library
import com.lotus.sametime.community.*;
import com.lotus.sametime.core.comparch.*;
import com.lotus.sametime.core.constants.*;
import com.lotus.sametime.im.*;
import java.io.*;
import java.util.Hashtable;

public class StockBot implements Runnable, LoginListener, ImServiceListener,
		ImListener {

	private Thread botThread;
	private STSession m_session;
	private CommunityService m_communityService;
	private InstantMessagingService m_imService;

	// Used to record if a users ST Client is Rich Text Capable
	Hashtable richTextFlags = new Hashtable(); 

	public StockBot(String serverName, String botName, String botPassword) {
		try {
			m_session = new STSession("StockBot " + this);
		} catch (DuplicateObjectException doe) {
			doe.printStackTrace();
			return;
		}
		m_session.loadSemanticComponents();
		m_session.start();
		m_communityService = (CommunityService) m_session
				.getCompApi(CommunityService.COMP_NAME);
		m_communityService.addLoginListener(this);
		m_communityService.loginByPassword(serverName, botName, botPassword);
	}

	public void loggedIn(LoginEvent e) {
		System.out.println("Succesfully Logged In to Sametime Server");
		m_imService = (InstantMessagingService) m_session
				.getCompApi(InstantMessagingService.COMP_NAME);
		m_imService.registerImType(ImTypes.IM_TYPE_CHAT);
		m_imService.addImServiceListener(this);
	}

	public void loggedOut(LoginEvent e) {
		System.out.println("Logged Out Reason: x"
				+ Integer.toHexString(e.getReason()));
	}

	public synchronized void imReceived(ImEvent e) {
		e.getIm().addImListener(this);
		System.out.println(e.getIm().getPartner().getDisplayName()+ " - IM Received");

		// Inform the incoming client over the Sametime datachannel 
		// that we can process RichText
		byte[] data = null;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dataStream = new DataOutputStream(baos);
		try {
			dataStream.writeUTF("data");
			dataStream.writeUTF("richtext");
			dataStream.write(new byte[] { (byte) 0xEE });
			data = baos.toByteArray();
		} catch (IOException errorexception) {
			throw new AssertionError("sendDataMessage failed");
		}
		// The message is now formatted, send the data indicating we have Rich Text Support
		e.getIm().sendData(true, 27191, 0, data);

		// Send a greeting to the user who started a converation with the bot
		e.getIm().sendText(
						true,
						"Welcome "
						+ e.getIm().getPartner().getDisplayName()
						+ " to Stock Bot, please enter a stock symbol to see a chart..."
						+ "\nStock Chart provided by http://StockCharts.com");
	}

	public synchronized void dataReceived(ImEvent e) {

		//Monitor the datachannel so we can check if the users client supports Rich Text
		System.out.println(e.getIm().getPartner().getDisplayName()
				+ " - dataReceived m_dataType=" + e.getDataType());

		String userId = e.getIm().getPartner().getId().getId();
		if (richTextFlags.get(userId) == null) {
			if (e.getDataType() == 27191)
				richTextFlags.put(userId, Boolean.TRUE);
			else if (e.getDataType() != 1) {
				richTextFlags.put(userId, Boolean.FALSE);
				/* 	A dataType of 1 is used to signfy the user has
				 *  started/stopped typing, so we do not want to set the client 
				 *  to no rich text if we receive that dataType
				 */
			}
		}

	}

	public void imClosed(ImEvent e) {
		System.out.println(e.getIm().getPartner().getName() + " - imClosed");

		String userId = e.getIm().getPartner().getId().getId();
		if (!richTextFlags.isEmpty())
			// Clear the tracking value for a users Rich Text support 
			// as they may connect with a different client in the future
			richTextFlags.remove(userId);
	}

	public void imOpened(ImEvent e) {
	//We don't initiate the chats, so no code needed here	
	}

	public void openImFailed(ImEvent e) {
		//We don't open the chat window, so no openIMFailed events will happen
	}

	public synchronized void textReceived(ImEvent e) {

		boolean CanReceiveRichText = false;

		/* Remove all the incoming removes any HTML formatting
		 * If the users client supports Rich Text ie HTML, it will contain
		 * HTML tags etc.  Remove all the tags with a regular expression
		 */
		String messageText = e.getText().replaceAll("\\<.*?\\>", "");

		System.out.println(e.getIm().getPartner().getName() + " - "
				+ messageText);

		// We've got the message, now we can do what we want with it,
		// if this was a directory bot, we could put the code here that goes 
		// and retrieves the directory data for example
		String userId = e.getIm().getPartner().getId().getId();
		Boolean flag = null;
		if (!richTextFlags.isEmpty())
			flag = (Boolean) richTextFlags.get(userId);
		if (flag != null)
			CanReceiveRichText = flag.booleanValue();

		if (!CanReceiveRichText) {
			// The users client is not enabled for Rich Text, so notify them
			// accordingly.  In this case we return a link to the chart on the web
			e.getIm().sendText(
							true,
							"I'm sorry, your Sametime Client doesn't support Rich Text"
							+ ", so I am unable to send you the chart image.  You can view the stock chart "
							+ "however at the following URL:\nhttp://stockcharts.com/c-sc/sc?s="
							+ messageText
							+ "&p=D&b=5&g=0&i=t98037453280&r=4220");
		} else {
			/* The users client supports rich text (basic HTML) 
			 * so we can return rich text to the user.  
			 * Sametime does not support all HTML, such as IFrames, javascript etc.
			 * for excurity reasons.
			 */
			e.getIm().sendText(
							true,
							"<span style=\"font-size:8pt;font-family:Tahoma;color:#000000;font-style:normal;\" class=\"left\">"
							+ "Stock Price for <b>"
							+ messageText
							+ "</b><BR><IMG SRC='http://stockcharts.com/c-sc/sc?s="
							+ messageText
							+ "&p=D&b=5&g=0&i=t98037453280&r=4220' height='431px'></span>");
		}
	}

	public void start() {
		if (botThread == null) {
			botThread = new Thread(this, "StockBotThread");
			botThread.start();
		}
	}

	public void run() {
		Thread myThread = Thread.currentThread();
		while (botThread == myThread) {
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
			}
		}
	}

	public static void main(String[] args) {
		if (args.length != 3) {
			System.out
					.println("Usage: StockBot serverName botName botPassword");
			System.exit(0);
		}
		StockBot stockBot = new StockBot(args[0], args[1], args[2]);
		stockBot.start();
	}
}
